#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.14
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import fs from 'fs';
import { program } from 'commander';
import chalk from 'chalk';
import sh from 'shelljs';
import to from 'await-to-done';
import { input, editor } from '@inquirer/prompts';
import { getIsGitProject, getGitRevParse } from '@gitmars/git';
import lang from './common/local.mjs';
import { defaults } from './common/global.mjs';

const { t } = lang;
const { green, red } = chalk;
if (!getIsGitProject()) {
  sh.echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
const { root } = getGitRevParse();
program.name("gitm init").usage("").description(t("Set configuration items for gitmars")).action(async () => {
  let key;
  for (key in defaults) {
    let data = defaults[key];
    if (["master", "develop", "release", "bugfix", "support"].includes(key)) {
      [, data = defaults[key]] = await to(
        input({
          message: t("Enter {branch} branch name", {
            branch: key
          }),
          default: defaults[key],
          transformer: (val) => val.trim(),
          validate: (val) => /^\w+$/.test(val) ? true : t("Enter the available names")
        })
      );
    } else if (key === "user") {
      [, data = key] = await to(
        input({
          message: t("Enter the Git username"),
          transformer: (val) => val.trim(),
          validate: (val) => val === "" || /^\w+$/.test(val) ? true : t("Enter the available names")
        })
      );
    } else if (key === "email") {
      [, data = key] = await to(
        input({
          message: t("Enter the Git email address"),
          transformer: (val) => val.trim(),
          validate: (val) => val === "" || /^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$/.test(val) ? true : t("Enter the correct email")
        })
      );
    } else if (key === "nameValidator") {
      [, data = key] = await to(
        input({
          message: t("Enter the branch name naming convention"),
          transformer: (val) => val.trim()
        })
      );
    } else if (key === "descriptionValidator") {
      [, data = key] = await to(
        input({
          message: t("Enter commit message rules"),
          transformer: (val) => val.trim()
        })
      );
    } else if (key === "msgTemplate") {
      [, data = defaults[key]] = await to(
        input({
          message: t("Enter the message template"),
          default: defaults[key],
          transformer: (val) => val.trim()
        })
      );
    } else if (key === "apolloConfig") {
      [, data = defaults[key]] = await to(
        editor({
          message: t("Enter apollo configuration"),
          default: `{
    "configServerUrl": "",
    "appId": "",
    "clusterName": "",
    "namespaceName": [],
    "apolloEnv": "",
    "token": ""
}`,
          validate: (val) => {
            try {
              val = JSON.parse(val);
              return true;
            } catch (e) {
              return t("Enter json");
            }
          }
        })
      );
    } else if (key === "hooks") {
      [, data = defaults[key]] = await to(
        editor({
          message: t("Enter hooks configuration"),
          default: `{
    "pre-commit": "",
    "pre-push": ""
}`,
          validate: (val) => {
            try {
              val = JSON.parse(val);
              return !!val;
            } catch (e) {
              return t("Enter json");
            }
          }
        })
      );
    } else if (key === "api") {
      [, data = defaults[key]] = await to(
        input({
          message: t("Enter the query user permission interface"),
          transformer: (val) => val.trim(),
          validate: (val) => val === "" || /^https?:\/\/[\S]*$/.test(val) ? true : t("Enter the URL")
        })
      );
    } else if (key === "gitHost") {
      [, data = defaults[key]] = await to(
        input({
          message: t("Enter the git URL"),
          transformer: (val) => val.trim(),
          validate: (val) => val === "" || /^https?:\/\/[\S]*$/.test(val) ? true : t("Enter the URL")
        })
      );
    } else if (key === "gitID") {
      [, data = defaults[key]] = await to(
        input({
          message: t("Enter the git project ID, currently only gitlab is supported"),
          transformer: (val) => val.trim(),
          validate: (val) => val === "" || /^\d+$/.test(val) ? true : t("Enter the URL")
        })
      );
    }
    defaults[key] = data;
  }
  try {
    defaults.apolloConfig = JSON.parse(defaults.apolloConfig);
    if (!defaults.apolloConfig.configServerUrl || !defaults.apolloConfig.token) {
      defaults.apolloConfig = "";
    }
  } catch (e) {
    defaults.apolloConfig = "";
  }
  try {
    let valid = false;
    defaults.hooks = JSON.parse(defaults.hooks);
    hooks:
      for (const k in defaults.hooks) {
        if (defaults.hooks[k]) {
          valid = true;
          break hooks;
        }
      }
    if (!valid)
      defaults.hooks = "";
  } catch (e) {
    defaults.hooks = "";
  }
  sh.echo(green(t("gitmars configured successfully")));
  fs.writeFileSync(root + "/.gitmarsrc", JSON.stringify(defaults, null, 4), "utf-8");
  fs.chmodSync(root + "/.gitmarsrc", 493);
});
program.parse(process.argv);
